<?php
/** @file
 * Infusionsoft API
 */


/**
 * Find an infusionsoft contact id by drupal username.
 *
 * @param object $user
 *   drupal user object (optional)
 *   defaults to logged in user.
 *
 * @return int|bool
 *   Returns the first contact id for the matching the email address provided by
 *   the username. If there is no username, it returns FALSE. Caches
 *   infusionsoft ids, for less hits to infusionsoft database.
 */
function infusionsoft_contactid_by_user($user = NULL) {

  return $contact_id;
}

/**
 * Find an infusionsoft contact id by email address.
 *
 * @param string $user_email
 *   Email address to look up. (optional)
 *   Defaults to the logged in user.
 *
 * @param bool $return_all
 *   If TRUE will return all contacts with an email address(optional)
 *   defaults to FALSE
 *
 *  @return bool
 *    If $return_all is TRUE, returns an array of contacts matching the email
 *    address. If FALSE it will return a integer contact_id
 */
function infusionsoft_contactid_by_email($user_email = NULL, $return_all = FALSE) {

  return $return_all ? $contact_ids : $contact_id;

}

/**
 * Push a drupal user update to Infusionsoft
 *
 *  NOTE: Does not update email addresses, if user is not already stored in the infusionsoft database.
 *  NOTE: You cannot use this function to 'Zero out' data. For instance if you set 'Email' => '' it will not update the data.
 *
 * @param array $contact_data
 *   A keyed array which maps to Infusionsoft data tables.
 *      Example:
 *        $contact_data = array(
 *          'FirstName' => 'John',
 *          'LastName'  => 'Doe',
 *          'Email'     => 'JDoe@email.com',
 *          'Username'	=> 'johndoe'
 *        );
 * @param object $user
 *   a drupal user object.
 *
 * @param array $options
 *   an (optional) array of any of the following options:
 *     'opt_in' -  TRUE (default)
 *       Only effects new additions to infusionsoft, will opt user in to
 *        marketing emails.
 *     'opt_reason' -  'Drupal Auto Opt In' (default)
 *        Only effects new additions to infusionsoft when opt in is enabled.
 *        Will add a custom reason to why an infusionsoft user is opting in.
 *     'no_email_update' - FALSE (default)
 *       force a contact to be updated if there is no supplied email address
 *       or drupal user connection.
 *
 * @return int
 *   returns the id number of the new contact record, or the id number of
 *   the existing record.
 */
function infusionsoft_update_drupal_user($contact_data, $user, $options = array()) {


    // lets return a contact_id
    return $contact_id;
}

/**
 * Update an infusionsoft contact. Add new contact if none exists.
 *
 *  NOTE: Does not update email addresses, if user is not already stored in the infusionsoft database.
 *  NOTE: You cannot use this function to 'Zero out' data. For instance if you set 'Email' => '' it will not update the data.
 *
 * @param array $contact_data
 *   A keyed array which maps to Infusionsoft data tables
 *   Example:
 *     $contact_data = array(
 *       'FirstName' => 'John',
 *       'LastName'  => 'Doe',
 *       'Email'     => 'JDoe@email.com',
 *       'Username'	=> 'johndoe'
 *     );
 *
 * @param int $contact_id
 *   (optional) - if set this will update a specific contact id, this is ideal
 *   because if we don't know an infusionsoft contact id we have to lookup the id.
 *
 * @param array $options
 *   (optional) - an array of any of the following options:
 *
 *   'opt_in' -  TRUE (default)
 *     Only affects new additions to infusionsoft, will opt user in to marketing emails.
 *
 *   'opt_reason' -  'Drupal Auto Opt In' (default)
 *     Only affects new additions to infusionsoft when opt in is enabled.
 *     Will add a custom reason to why an infusionsoft user is opting in.
 *
 *   'no_email_update' - FALSE (default)
 *     Force a contact to be updated if there is no supplied email address or drupal user connection.
 *
 *   'dup_check_type' - 'Email' (default)
 *     Determines how to consider a duplicate record, can be 'Email','EmailAndName' or 'EmailAndNameAndCompany'.
 *
 * @return int|bool
 *   Returns the id number of the new contact record, or the id number of the
 *   existing record. Returns FALSE if there is no data.
 */
function infusionsoft_update_contact($contact_data, $contact_id = NULL, $options = array()) {

  return $contact_id;
}

/**
 * Send an e-mail to the given contact(s)
 *
 * @param array $contact_ids
 *   an array of (int) infusionsoft contact ids. OR a single $contact_id
 *
 * @param string $subject
 *   The email subject.
 *
 * @param string $text_body
 *   email body for text users
 *
 * @param string $html_body
 *   email body for html users
 *
 * @param string $type
 *   the type of email you are sending
 *   - options
 *     - HTML - creates an html email
 *     - Text - creates a text based email
 *     - Multipart - creates an email which is both HTML and Text
 *
 * @param string $from
 *   (optional) - the email address this template will be sent from. (defaults to owner)
 *
 * @param string $to
 *   (optional) - the email address this template will send to.
 *   This is typically the merge field "~Contact.Email~"
 *
 * @param string $cc
 *   The email address(es) this template will carbon copy. Multiples separated by comma
 *
 * @param string $bcc
 *   The email address(es) this template will blind carbon copy. Multiples separated by comma
 *
 * @return bool
 *   TRUE if email has been sent
 */
function infusionsoft_send_email($contact_ids, $subject, $text_body = '', $html_body = '', $type = 'text', $from = '~Owner.Email~', $to = '~Contact.Email~', $cc = '', $bcc = '') {

  return $email_sent ? TRUE : FALSE;
}

/**
 * Update a contact's follow up campaign.
 *
 * @param int $contact_id
 *  An infusionsoft contact id.
 *
 * @param int $followup_id
 *   An infusionsoft campaign id.
 *
 * @param $type
 *   The type of update
 *   - options
 *     - assign - assigns a contact to a follow up campaign
 *     - remove - removes a contact from a follow up campaign
 *     - pause - pauses a follow up campaign for a contact
 *     - resume - resumes a paused campaign
 *
 * @return bool
 *   TRUE if it succeeded or FALSE if it did not.
 */
function infusionsoft_follow_up_update($contact_id, $followup_id, $type = 'assign') {

  return $success ? TRUE : FALSE;
}

/**
 * Run an action set on a contact.
 *
 * @param int $contact_id
 *   An infusionsoft contact id.
 *
 * @param
 *   $actionset_id - an infusionsoft action set id.
 *
 * @return bool
 *   TRUE if it succeeded or FALSE if it did not.
 */
function infusionsoft_run_actionset($contact_id, $actionset_id) {

  return $success ? TRUE : FALSE;
}

/**
 * Look up a contact group name from a group id.
 *
 * @param int $contact_group_id
 *   id of the contact group to look up the name of
 *
 * @return array
 *   array of fields
 */
function infusionsoft_get_contact_group_name($contact_group_id) {

  return $contact_group_name;
}

/**
 * Runs an infusionsoft findByField query as many times as needed to get all results.
 *
 * This is needed because Infusionsoft caps findByfield queries at 1000 responses.
 * This operation is particularly useful when doing a lookup on a foreign key, for
 * example looking up all Invoices related to a given contact.
 *
 * @param
 *   $table - the infusionsoft table to query
 * @param
 *   $query_data - the data to look for
 * @param
 *   $return_fields - the fields to return from the query
 *
 * @return
 *    An array of arrays in the format:
 *      array(
 *				array(
 *					'ReturnField1' => something,
 *					'ReturnField2' => something else,
 *					...
 *				)
 *				...
 *      );
 *
 * NOTE: Use this very sparingly, if you have 1000s of records this can have a performance impact.
 *   If you use this function frequently you will want to cache it!
 *
 */
function infusionsoft_findbyfield_all($table, $field_name, $field_value, $selected_fields) {

  // returns an array of arrays;
  return $array;
}

/**
 * Runs an infusionsoft query as many times as needed to get all results.
 *
 * This is needed because Infusionsoft caps queries at 1000 responses.
 *
 * @param
 *   $table - the infusionsoft table to query
 * @param
 *   $query_data - the data to look for
 * @param
 *   $return_fields - the fields to return from the query
 *
 * @return
 *    An array of arrays in the format:
 *      array(
 *				array(
 *					'ReturnField1' => something,
 *					'ReturnField2' => something else,
 *					...
 *				)
 *				...
 *      );
 *
 * NOTE: Use this very sparingly, if you have 1000s of records this can have a performance impact.
 *   If you use this function frequently you will want to cache it!
 */
function infusionsoft_query_all($table, $query_data, $selected_fields) {

  // returns an array of arrays;
  return $array;
}

/**
 * Primay callback function to send an API request to infusionsoft.
 *
 * Call an infusionsoft request, followed by the proper parameters.
 *
 * @param
 *   $request_type - the API function to call
 * @return
 *   FALSE if there is no api key
 *   OR Nothing if there is a problem with the query
 *   OR the return values for the given service and method sent via xml-rpc request.
 *   See http://help.infusionsoft.com/developers/services-methods for more info.
 *
 */
function infusionsoft_send_request($request_type = 'ContactService.load') {

  // see above, return depends on query
  return $return;
}